#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-12-28
# File  : fritzbox_smarthome_temperature.py (check plugin)
#

# 2024-06-17: modified import for CMK 2.3

from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import Result, Service, State, register
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import CheckResult, DiscoveryResult
from cmk.base.plugins.agent_based.utils.fritzbox_smarthome import AvmSmartHomeDevice
from cmk.base.plugins.agent_based.utils.temperature import check_temperature
from cmk.plugins.lib.temperature import _render_temp_with_unit


def discovery_fritzbox_smarthome_temperature_single(
        section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> DiscoveryResult:
    if isinstance(section, AvmSmartHomeDevice):
        if section.temperature:
            yield Service()


def discovery_fritzbox_smarthome_temperature_multiple(
        section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> DiscoveryResult:
    if not isinstance(section, AvmSmartHomeDevice):
        for device_id, device in section.items():
            if device.temperature:
                yield Service(item=str(device_id))


def check_fritzbox_smarthome_temperature_single(
        params, section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> CheckResult:
    if not isinstance(section, AvmSmartHomeDevice) or not section.temperature:
        return

    if section.temperature.celsius:
        yield from check_temperature(
            reading=section.temperature.celsius,
            params=params,
        )
    if section.temperature.offset:
        temp_sensor = section.temperature.celsius + section.temperature.offset * -1
        _details = (
            f'Temperature measured at the sensor'
            f': {_render_temp_with_unit(temp_sensor, params.get("output_unit", "c"))}'
        )
        _summary = f'At the sensor: {_render_temp_with_unit(temp_sensor, params.get("output_unit", "c"))}'
        yield Result(state=State.OK, summary=_summary, details=_details)

        yield Result(
            state=State.OK,
            summary=f'Offset: {_render_temp_with_unit(section.temperature.offset, params.get("output_unit", "c"))}',
            details=f'Temperature offset: '
                    f'{_render_temp_with_unit(section.temperature.offset, params.get("output_unit", "c"))}'
        )


def check_fritzbox_smarthome_temperature_multiple(
        item, params, section: AvmSmartHomeDevice | Dict[str, AvmSmartHomeDevice]
) -> CheckResult:
    if isinstance(section, Dict):
        try:
            yield from check_fritzbox_smarthome_temperature_single(params, section[item])
        except KeyError:
            return


register.check_plugin(
    name='fritzbox_smarthome_temperature_single',
    service_name='Temperature',
    sections=['fritzbox_smarthome'],
    discovery_function=discovery_fritzbox_smarthome_temperature_single,
    check_function=check_fritzbox_smarthome_temperature_single,
    check_ruleset_name='temperature_single',
    check_default_parameters={}
)

register.check_plugin(
    name='fritzbox_smarthome_temperature',
    service_name='Smarthome Temperature %s',
    sections=['fritzbox_smarthome'],
    discovery_function=discovery_fritzbox_smarthome_temperature_multiple,
    check_function=check_fritzbox_smarthome_temperature_multiple,
    check_ruleset_name='temperature',
    check_default_parameters={}
)
